@@ -1,21 +1,24 @@
 --[[
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+This file is part of sysdig.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
 --]]
 
 -- Chisel description
-description = "Show the top process defined by the highest CPU utilization. This chisel is compatable with containers using the sysdig -pc or -pcontainer argument, otherwise no container information will be shown."
+description = "Show the top process defined by the highest CPU utilization. This chisel is compatible with containers using the sysdig -pc or -pcontainer argument, otherwise no container information will be shown."
 short_description = "Top processes by CPU usage"
 category = "CPU Usage"
 
@@ -27,7 +30,6 @@ terminal = require "ansiterminal"
 
 grtable = {}
 islive = false
-cpustates = {}
 fkeys = {}
 local print_container = false
 
@@ -50,7 +52,7 @@ function on_init()
 	-- Print container info as well
 	if print_container then
 		-- Modify host pid column name and add container information
-		vizinfo.key_fld = {"proc.name", "proc.pid", "thread.vtid", "container.name"}
+		vizinfo.key_fld = {"proc.name", "proc.pid", "proc.vpid", "container.name"}
 		vizinfo.key_desc = {"Process", "Host_pid", "Container_pid", "container.name"}
 	end
 
@@ -61,11 +63,10 @@ function on_init()
 
 	-- Request the fields we need
 	fvalue = chisel.request_field(vizinfo.value_fld)
-	fnext = chisel.request_field("evt.arg.next")
-	fnextraw = chisel.request_field("evt.rawarg.next")
-	
-	chisel.set_filter("evt.type=switch")
+	fcpu = chisel.request_field("thread.cpu")
 	
+	chisel.set_filter("evt.type=procinfo")
+
 	return true
 end
 
@@ -82,18 +83,11 @@ function on_capture_start()
 		end
 	end
 
-	ncpus = sysdig.get_machine_info().num_cpus
-
-	for j = 1, ncpus do
-		cpustates[j] = {0, 0, 0, ""}
-	end
-
 	return true
 end
 
 -- Event parsing callback
 function on_event()
-
 	local key = nil
 	local kv = nil
 
@@ -110,32 +104,14 @@ function on_event()
 		end
 	end
 
-	value = evt.field(fvalue)
-	cpuid = evt.get_cpuid() + 1
+	local cpu = evt.field(fcpu)
 
-	if key ~= nil and value ~= nil and value > 0 then
-		thissec = value - cpustates[cpuid][3]
-		if thissec < 0 then
-			thissec = 0
-		end
-
-		if grtable[key] == nil then
-			grtable[key] = thissec
-		else
-			grtable[key] = grtable[key] + thissec
-		end
-		
-		cpustates[cpuid][1], cpustates[cpuid][2] = evt.get_ts()
-	end
-
-	if evt.field(fnext) ~= "" .. evt.field(fnextraw) then
-		cpustates[cpuid][4] = evt.field(fnext)
+	if grtable[key] == nil then
+		grtable[key] = cpu * 10000000
 	else
-		cpustates[cpuid][4] = nil
+		grtable[key] = grtable[key] + (cpu * 10000000)
 	end
 
-	cpustates[cpuid][3] = 0
-
 	return true
 end
 
@@ -146,22 +122,6 @@ function on_interval(ts_s, ts_ns, delta)
 		terminal.moveto(0, 0)
 	end
 	
-	for cpuid = 1, ncpus do
-		if cpustates[cpuid][1] ~= 0 then
-			cpustates[cpuid][3] = 1000000000 - cpustates[cpuid][2]
-
-			key = cpustates[cpuid][4]
-
-			if key ~= nil and value ~= nil and value > 0 then
-				if grtable[key] == nil then
-					grtable[key] = cpustates[cpuid][3]
-				else
-					grtable[key] = grtable[key] + cpustates[cpuid][3]
-				end
-			end
-		end
-	end
-	
 	print_sorted_table(grtable, ts_s, 0, delta, vizinfo)
 	
 	-- Clear the table
