@@ -26,7 +26,7 @@ view_info =
 	tags = {"Default", "wsysdig"},
 	view_type = "table",
 	applies_to = {"", "container.id", "proc.pid", "thread.nametid", "proc.name", "thread.tid", "fd.sport", "fd.sproto", "evt.res", "k8s.pod.id", "k8s.rc.id", "k8s.rs.id", "k8s.svc.id", "k8s.ns.id", "marathon.app.id", "marathon.group.name", "mesos.task.id", "mesos.framework.name"},
-	filter = "fd.type=file or fd.type=directory and fd.name!=''",
+	filter = "(fd.type=file or fd.type=directory) and fd.name!=''",
 	use_defaults = true,
 	drilldown_target = "files",
 	columns = 
