@@ -42,6 +42,7 @@ limitations under the License.
 #include "fields_info.h"
 #include "table.h"
 #include "utils.h"
+#include "plugin_utils.h"
 
 #ifdef _WIN32
 #include "win32/getopt.h"
@@ -62,6 +63,7 @@ limitations under the License.
 #define MOUSE_CAPABLE_TERM_COMPAT "xterm-1002"
 
 static bool g_terminate = false;
+static bool g_plugin_input = false;
 static void usage();
 
 //
@@ -69,7 +71,29 @@ static void usage();
 //
 static void signal_callback(int signal)
 {
-	g_terminate = true;
+	if(g_plugin_input)
+	{
+		//
+		// Input plugins can get stuck at any point.
+		// When we are using one, check again in few seconds and force a quit
+		// if we are stuck.
+		//
+		if(g_terminate == true)
+		{
+			exit(0);
+		}
+		else
+		{
+			g_terminate = true;
+#ifndef _WIN32
+			alarm(2);
+#endif
+		}
+	}
+	else
+	{
+		g_terminate = true;
+	}
 }
 
 //
@@ -196,7 +220,7 @@ static void add_chisel_dirs(sinsp* inspector)
 	//
 	// Add the default chisel directory statically configured by the build system
 	//
-	chisel_add_dir(SYSDIG_INSTALLATION_DIR CHISELS_INSTALLATION_DIR, false);
+	chisel_add_dir(SYSDIG_CHISELS_DIR, false);
 
 	//
 	// Add the directories configured in the SYSDIG_CHISEL_DIR environment variable
@@ -410,6 +434,8 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 #ifdef HAS_CHISELS
 		add_chisel_dirs(inspector);
 #endif
+		add_plugin_dirs(SYSDIG_PLUGINS_DIR);
+		init_plugins(inspector);
 
 		//
 		// Parse the args
@@ -891,7 +917,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 					}
 					else
 					{
-						if(system("modprobe " PROBE_NAME " > /dev/null 2> /dev/null"))
+						if(system("modprobe " DRIVER_NAME " > /dev/null 2> /dev/null"))
 						{
 							fprintf(stderr, "Unable to load the driver\n");
 						}
@@ -992,6 +1018,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 			}
 			delete mesos_api;
 			mesos_api = 0;
+#endif // MINIMAL_BUILD
 
 			if(output_type == sinsp_table::OT_JSON)
 			{
@@ -1001,7 +1028,6 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 					printf("{\"progress\": 0},\n");
 				}
 			}
-#endif // MINIMAL_BUILD
 
 			//
 			// Start the capture loop
