@@ -57,6 +57,7 @@ limitations under the License.
 #endif
 
 static bool g_terminate = false;
+static bool g_terminating = false;
 static bool g_plugin_input = false;
 #ifdef HAS_CHISELS
 vector<sinsp_chisel*> g_chisels;
@@ -72,11 +73,11 @@ static void signal_callback(int signal)
 	if(g_plugin_input)
 	{
 		//
-		// Input plugins can get stuck at any point.
+		// Input plugins can get stuck in next().
 		// When we are using one, check again in few seconds and force a quit
 		// if we are stuck.
 		//
-		if(g_terminate == true)
+		if(g_terminate == true && g_terminating == false)
 		{
 			exit(0);
 		}
@@ -84,7 +85,7 @@ static void signal_callback(int signal)
 		{
 			g_terminate = true;
 #ifndef _WIN32
-			alarm(2);
+			alarm(3);
 #endif
 		}
 	}
@@ -120,7 +121,7 @@ static void usage()
 "                    they must be specified in the command line after the name.\n"
 " -cl, --list-chisels\n"
 "                    lists the available chisels. Sysdig looks for chisels in the\n"
-"                    following directories: ./chisels, ~/.chisels, /usr/share/sysdig/chisels.\n"
+"                    following directories: ./chisels, ~/.chisels, " SYSDIG_CHISELS_DIR ".\n"
 #endif
 " -C <file_size>, --file-size=<file_size>\n"
 "                    Before writing an event, check whether the file is\n"
@@ -185,20 +186,30 @@ static void usage()
 "                    If no data format is specified, this can be used with -W flag to\n"
 "                    create a ring buffer of events.\n"
 " -h, --help         Print this page\n"
-" -I <inputname>[:<inputargs>], --input <inputname>[:<inputargs>]\n"
-"                    (PREVIEW feature, subject to change)\n"
-"                    capture events using the plugin with name inputname, passing to the \n"
-"                    plugin the inputargs string as parameters.\n"
-"                    The format of inputargs is controller by the plugin, refer to each\n"
+" -H <pluginname>[:<initconfig>], --plugin <pluginname>[:<initconfig>]\n"
+"                    registers a plugin, using the passed init config if present.\n"
+"                    The format of initconf is controlled by the plugin, refer to each\n"
 "                    plugin's documentation to learn about it.\n"
-"                    The event sources available for capture vary depending on which \n"
-"                    plugins have been installed. You can list the plugins that have been \n"
-"                    loaded by using the -Il flag.\n"
-" -Il\n"
-"                    (PREVIEW feature, subject to change)\n"
-"                    lists the loaded plugins. Sysdig looks for plugins in the directories \n"
+"                    A path can also be used as pluginname.\n"
+" -I <pluginname>[:<openparams>], --input <pluginname>[:<openparams>]\n"
+"                    set a previously registered plugin as input plugin,\n"
+"                    capturing events using it and passing the \n"
+"                    openparams string as open parameters.\n"
+"                    Only a single source plugin can be registered.\n"
+"                    If no plugins were registered, any found plugin in the directories\n"
 "                    specified by ;-separated environment variable SYSDIG_PLUGIN_DIR and\n"
-"     				 in /usr/share/sysdig/plugins.\n"
+"                    in " SYSDIG_PLUGINS_DIR " is registered; then use the provided one as input source.\n"
+"                    The format of openparams is controlled by the plugin, refer to each\n"
+"                    plugin's documentation to learn about it.\n"
+"                    See https://falco.org/docs/plugins/plugin-api-reference/#ss-plugin-t-plugin-init-const-char-config-int32-t-rc-required-yes\n"
+"                    and https://falco.org/docs/plugins/plugin-api-reference/#ss-instance-t-plugin-open-ss-plugin-t-s-const-char-params-int32-t-rc-required-yes for more infos.\n"
+"                    The event sources available for capture vary depending on which \n"
+"                    plugins have been installed.\n"
+" -Il\n"
+"                    lists the loaded plugins. If no plugin has been registered through '-H',\n"
+"                    Sysdig looks for plugins in the directories \n"
+"                    specified by ;-separated environment variable SYSDIG_PLUGIN_DIR and\n"
+"                    in " SYSDIG_PLUGINS_DIR ".\n"
 #ifdef HAS_CHISELS
 " -i <chiselname>, --chisel-info <chiselname>\n"
 "                    Get a longer description and the arguments associated with\n"
@@ -250,6 +261,11 @@ static void usage()
 " -n <num>, --numevents=<num>\n"
 "                    Stop capturing after <num> events\n"
 " --page-faults      Capture user/kernel major/minor page faults\n"
+" --plugin-config-file\n"
+"                    Load the plugin configuration from a Falco-compatible yaml file.\n"
+"                    Do not mix this option with the '-H' or '-I' options: it is unsupported.\n"
+"                    See the plugin section in https://falco.org/docs/configuration/ for\n"
+"                    additional information\n"
 " -P, --progress     Print progress on stderr while processing trace files\n"
 " -p <output_format>, --print=<output_format>\n"
 "                    Specify the format to be used when printing the events.\n"
@@ -345,6 +361,10 @@ static void usage()
 "   $ sysdig proc.name=cat and evt.type=open\n\n"
 " Print the name of the files opened by cat\n"
 "   $ sysdig -p\"%%evt.arg.name\" proc.name=cat and evt.type=open\n\n"
+" Register any found plugin and use dummy as input source passing to it open params\n"
+"   $ sysdig -I dummy:'{\"start\":1,\"maxEvents\":10}'\n\n"
+" Load and register dummy source plugin passing to it init config and open params\n"
+"   $ sysdig -H dummy:'{\"jitter\":50}' -I dummy:'{\"start\":1,\"maxEvents\":10}'\n\n"
     );
 }
 
@@ -444,7 +464,7 @@ static void add_chisel_dirs(sinsp* inspector)
 	//
 	// Add the default chisel directory statically configured by the build system
 	//
-	chisel_add_dir(SYSDIG_INSTALLATION_DIR CHISELS_INSTALLATION_DIR, false);
+	chisel_add_dir(SYSDIG_CHISELS_DIR, false);
 
 	//
 	// Add the directories configured in the SYSDIG_CHISEL_DIR environment variable
@@ -554,10 +574,9 @@ static void parse_chisel_args(sinsp_chisel* ch, sinsp* inspector, int optind, in
 static void free_chisels()
 {
 #ifdef HAS_CHISELS
-	for(vector<sinsp_chisel*>::iterator it = g_chisels.begin();
-		it != g_chisels.end(); ++it)
+	for(auto & g_chisel : g_chisels)
 	{
-		delete *it;
+		delete g_chisel;
 	}
 
 	g_chisels.clear();
@@ -567,9 +586,9 @@ static void free_chisels()
 static void chisels_on_capture_start()
 {
 #ifdef HAS_CHISELS
-	for(uint32_t j = 0; j < g_chisels.size(); j++)
+	for(auto & g_chisel : g_chisels)
 	{
-		g_chisels[j]->on_capture_start();
+		g_chisel->on_capture_start();
 	}
 #endif
 }
@@ -577,10 +596,9 @@ static void chisels_on_capture_start()
 static void chisels_on_capture_end()
 {
 #ifdef HAS_CHISELS
-	for(vector<sinsp_chisel*>::iterator it = g_chisels.begin();
-		it != g_chisels.end(); ++it)
+	for(auto & g_chisel : g_chisels)
 	{
-		(*it)->on_capture_end();
+		g_chisel->on_capture_end();
 	}
 #endif
 }
@@ -688,7 +706,8 @@ captureinfo do_inspect(sinsp* inspector,
 	bool print_progress,
 	sinsp_filter* display_filter,
 	vector<summary_table_entry> &summary_table,
-	sinsp_evt_formatter* formatter)
+	sinsp_evt_formatter* syscall_evt_formatter,
+	sinsp_evt_formatter* plugin_evt_formatter)
 {
 	captureinfo retval;
 	int32_t res;
@@ -701,6 +720,10 @@ captureinfo do_inspect(sinsp* inspector,
 		do_flush = true;
 	}
 
+	// This changes between syscall_evt_formatter and
+	// plugin_evt_formatter based on the event type.
+	sinsp_evt_formatter *formatter = syscall_evt_formatter;
+
 	//
 	// Loop through the events
 	//
@@ -712,6 +735,10 @@ captureinfo do_inspect(sinsp* inspector,
 			// End of capture, either because the user stopped it, or because
 			// we reached the event count specified with -n.
 			//
+			if(g_terminate)
+			{
+				g_terminating = true;
+			}
 			handle_end_of_file(inspector, print_progress, reset_colors, formatter);
 			break;
 		}
@@ -751,6 +778,8 @@ captureinfo do_inspect(sinsp* inspector,
 			throw sinsp_exception(inspector->getlasterr().c_str());
 		}
 
+		formatter = (ev->get_type() == PPME_PLUGINEVENT_E ? plugin_evt_formatter : syscall_evt_formatter);
+
 		if (duration_start == 0)
 		{
 			duration_start = ev->get_ts();
@@ -908,6 +937,60 @@ std::string escape_output_format(const std::string& s)
     return ss.str();
 }
 
+static void list_plugins(sinsp *inspector)
+{
+	// This will either register any found plugin or
+	// only plugins marked with '-H'
+	init_plugins(inspector);
+	auto plugins = inspector->get_plugins();
+	std::ostringstream os_dirs, os_info;
+
+	for(const plugin_dir_info& path : get_plugin_dirs())
+	{
+		os_dirs << path.m_dir << " ";
+	}
+
+	for(auto &p : plugins)
+	{
+		os_info << "Name: " << p->name() << std::endl;
+		os_info << "Description: " << p->description() << std::endl;
+		os_info << "Contact: " << p->contact() << std::endl;
+		os_info << "Version: " << p->plugin_version().as_string() << std::endl;
+
+		// Print schema
+		ss_plugin_schema_type schema_type;
+		auto schema = p->get_init_schema(schema_type);
+		os_info << "Init Config Schema: " << schema << std::endl;
+
+		if(p->type() == TYPE_SOURCE_PLUGIN)
+		{
+			auto splugin = dynamic_cast<sinsp_source_plugin *>(p.get());
+			os_info << "Type: source plugin" << std::endl;
+			os_info << "ID: " << splugin->id() << std::endl;
+			os_info << "Suggested Open Params:" << std::endl;
+			for(auto &oparam : splugin->list_open_params())
+			{
+				if(oparam.desc == "")
+				{
+					os_info << oparam.value << std::endl;
+				}
+				else
+				{
+					os_info << oparam.value << ": " << oparam.desc << std::endl;
+				}
+			}
+		}
+		else
+		{
+			os_info << "Type: extractor plugin" << std::endl;
+		}
+		os_info << std::endl;
+	}
+
+	printf("Plugin search paths are: %s\n", os_dirs.str().c_str());
+	printf("%lu Plugins Loaded:\n\n%s\n", plugins.size(), os_info.str().c_str());
+}
+
 //
 // ARGUMENT PARSING AND PROGRAM SETUP
 //
@@ -957,8 +1040,6 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 	string cri_socket_path;
 #endif
 	bool udig = false;
-	string inputname;
-	bool has_src_plugin = false;
 
 	// These variables are for the cycle_writer engine
 	int duration_seconds = 0;
@@ -987,6 +1068,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		{"filter-proclist", no_argument, 0, 0 },
 		{"seconds", required_argument, 0, 'G' },
 		{"help", no_argument, 0, 'h' },
+		{"plugin", required_argument, 0, 'H' },
 		{"input", required_argument, 0, 'I' },
 #ifdef HAS_CHISELS
 		{"chisel-info", required_argument, 0, 'i' },
@@ -1008,6 +1090,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 #endif // MINIMAL_BUILD
 		{"numevents", required_argument, 0, 'n' },
 		{"page-faults", no_argument, 0, 0 },
+		{"plugin-config-file", required_argument, 0, 0},
 		{"progress", required_argument, 0, 'P' },
 		{"print", required_argument, 0, 'p' },
 		{"quiet", no_argument, 0, 'q' },
@@ -1032,8 +1115,8 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 
 	if (isatty(fileno(stdout)))
 	{
-		output_format = "*%evt.num %evt.outputtime %evt.cpu \e[01;32m%proc.name\e[00m (\e[01;36m%proc.pid\e[00m.%thread.tid) %evt.dir \e[01;34m%evt.type\e[00m %evt.info";
-		output_format_plugin = "*%evt.num %evt.datetime.s [%evt.pluginname] %evt.plugininfo";
+		output_format = R"(*%evt.num %evt.outputtime %evt.cpu \e[01;32m%proc.name\e[00m (\e[01;36m%proc.pid\e[00m.%thread.tid) %evt.dir \e[01;34m%evt.type\e[00m %evt.info)";
+		output_format_plugin = R"(*%evt.num %evt.datetime.s [\e[01;32m%evt.pluginname\e[00m] %evt.plugininfo)";
 	} else
 	{
 		output_format = "*%evt.num %evt.outputtime %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.info";
@@ -1048,8 +1131,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 #ifdef HAS_CHISELS
 		add_chisel_dirs(inspector);
 #endif
-		add_plugin_dirs(SYSDIG_INSTALLATION_DIR);
-		register_plugins(inspector);
+		add_plugin_dirs(SYSDIG_PLUGINS_DIR);
 
 		//
 		// Parse the args
@@ -1059,7 +1141,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
                                         "C:"
                                         "dDEe:F"
                                         "G:"
-                                        "hI:i:jk:K:lLm:M:n:Pp:qRr:Ss:t:TU:uv"
+                                        "hH:I:i:jk:K:lLm:M:n:Pp:qRr:Ss:t:TU:uv"
                                         "W:"
                                         "w:xXz", long_options, &long_index)) != -1)
 		{
@@ -1154,61 +1236,43 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 					goto exit;
 				}
 				break;
+			case 'H':
+				{
+					string pluginname = optarg;
+					size_t cpos = pluginname.find(':');
+					string pgname = pluginname;
+					string pginitconf;
+					// Extract init config from string if present
+					if(cpos != string::npos)
+					{
+						pgname = pluginname.substr(0, cpos);
+						pginitconf = pluginname.substr(cpos + 1);
+					}
+					select_plugin_init(inspector, pgname, pginitconf);
+					break;
+				}
 			case 'I':
 				{
-					inputname = optarg;
+					string inputname = optarg;
 					if(inputname == "l")
 					{
-						std::list<sinsp_plugin::info> infos = sinsp_plugin::plugin_infos(inspector);
-						std::ostringstream os_dirs, os_info;
-
-						for(plugin_dir_info path : get_plugin_dirs()) {
-							os_dirs << path.m_dir << " ";
-						}
-
-						for(auto &info : infos)
-						{
-							os_info << "Name: " << info.name << std::endl;
-							os_info << "Description: " << info.description << std::endl;
-							os_info << "Contact: " << info.contact << std::endl;
-							os_info << "Version: " << info.plugin_version.as_string() << std::endl;
-
-							if(info.type == TYPE_SOURCE_PLUGIN)
-							{
-								os_info << "Type: source plugin" << std::endl;
-								os_info << "ID: " << info.id << std::endl;
-							}
-							else
-							{
-								os_info << "Type: extractor plugin" << std::endl;
-							}
-						}
-
-						printf("Plugin search paths are: %s\n", os_dirs.str().c_str());
-						printf("%lu Plugins Loaded:\n\n%s\n", infos.size(), os_info.str().c_str());
+						list_plugins(inspector);
 						delete inspector;
 						return sysdig_init_res(EXIT_SUCCESS);
 					}
 
-					has_src_plugin = true;
-
 					size_t cpos = inputname.find(':');
-					string pgname;
+					string pgname = inputname;
 					string pgpars;
+					// Extract open params from string if present
 					if(cpos != string::npos)
 					{
 						pgname = inputname.substr(0, cpos);
 						pgpars = inputname.substr(cpos + 1);
-						inspector->set_input_plugin(pgname);
-						inspector->set_input_plugin_open_params(pgpars);
-					}
-					else
-					{
-						inspector->set_input_plugin(inputname);
 					}
 
+					select_plugin_enable(pgname, pgpars);
 					g_plugin_input = true;
-					//print_progress = true;
 				}
 				break;
 #ifdef HAS_CHISELS
@@ -1351,7 +1415,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				inspector->set_hostname_and_port_resolution_mode(true);
 				break;
 			case 'r':
-				infiles.push_back(optarg);
+				infiles.emplace_back(optarg);
 #ifndef MINIMAL_BUILD
 				k8s_api = new string();
 				mesos_api = new string();
@@ -1487,6 +1551,10 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 						list_flds_markdown = true;
 					}
 
+					else if(optname == "plugin-config-file") {
+						g_plugin_input = parse_plugin_configuration_file(inspector, optarg);
+					}
+
 					else if (optname == "page-faults") {
 						page_faults = true;
 					}
@@ -1502,6 +1570,12 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			}
 		}
 
+		init_plugins(inspector);
+		if(g_plugin_input)
+		{
+			enable_source_plugin(inspector);
+		}
+
 #ifdef HAS_CAPTURE
 		if(!cri_socket_path.empty())
 		{
@@ -1528,7 +1602,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		// If we are dumping events to file, enable progress printing so we can give
 		// feedback to the user
 		//
-		if(outfile != "" && (infiles.size() != 0 || g_plugin_input == true))
+		if(!outfile.empty() && (!infiles.empty() || g_plugin_input))
 		{
 			print_progress = true;
 		}
@@ -1634,6 +1708,12 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			reset_colors = true;
 		}
 
+		output_format_plugin = escape_output_format(output_format_plugin);
+		if (output_format_plugin.find("\e") != std::string::npos)
+		{
+			reset_colors = true;
+		}
+
 #ifndef _WIN32
 		if(signal(SIGALRM, signal_callback) == SIG_ERR)
 		{
@@ -1646,13 +1726,14 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		//
 		// Create the event formatter
 		//
-		sinsp_evt_formatter formatter(inspector,
-					      (g_plugin_input ? output_format_plugin : output_format));
+		sinsp_evt_formatter syscall_evt_formatter(inspector, output_format);
+
+		sinsp_evt_formatter plugin_evt_formatter(inspector, output_format_plugin);
 
 		//
 		// Set output buffers len
 		//
-		if(!verbose && g_chisels.size() == 0)
+		if(!verbose && g_chisels.empty())
 		{
 			inspector->set_max_evt_output_len(80);
 		}
@@ -1662,9 +1743,9 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		//
 		if(filter_proclist_flag)
 		{
-			if(filter != "")
+			if(!filter.empty())
 			{
-				if(infiles.size() == 0)
+				if(infiles.empty())
 				{
 					fprintf(stderr, "--filter-proclist not supported with live captures.\n");
 					res.m_res = EXIT_FAILURE;
@@ -1681,10 +1762,10 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			}
 		}
 
-		for(uint32_t j = 0; j < infiles.size() || infiles.size() == 0; j++)
+		for(uint32_t j = 0; j < infiles.size() || infiles.empty(); j++)
 		{
 #ifdef HAS_FILTERING
-			if(filter.size() && !is_filter_display)
+			if(!filter.empty() && !is_filter_display)
 			{
 				inspector->set_filter(filter);
 			}
@@ -1695,7 +1776,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			// inspector, as that reads the process list.
 			for(auto &comm : suppress_comms)
 			{
-				if (!inspector->suppress_events_comm(comm.c_str()))
+				if (!inspector->suppress_events_comm(comm))
 				{
 					fprintf(stderr, "Could not add %s to the set of suppressed comms--did you specify more than %d values?\n",
 						comm.c_str(),
@@ -1708,7 +1789,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			//
 			// Launch the capture
 			//
-			if(infiles.size() != 0)
+			if(!infiles.empty())
 			{
 				initialize_chisels();
 
@@ -1745,17 +1826,14 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				}
 				else
 				{
-					if(has_src_plugin)
+
+					try
 					{
 						inspector->open("");
 					}
-					else
+					catch(const sinsp_exception& e)
 					{
-						try
-						{
-							inspector->open("");
-						}
-						catch(const sinsp_exception& e)
+						if (!g_plugin_input)
 						{
 							open_success = false;
 						}
@@ -1781,7 +1859,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 						}
 						else
 						{
-							if(system("modprobe " PROBE_NAME " > /dev/null 2> /dev/null"))
+							if(system("modprobe " DRIVER_NAME " > /dev/null 2> /dev/null"))
 							{
 								fprintf(stderr, "Unable to load the driver\n");
 							}
@@ -1916,7 +1994,8 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				print_progress,
 				display_filter,
 				summary_table,
-				&formatter);
+				&syscall_evt_formatter,
+				&plugin_evt_formatter);
 
 			duration = ((double)clock()) / CLOCKS_PER_SEC - duration;
 
